/*
 * This work is licensed under a Creative Commons 
 * Attribution-Noncommercial-Share Alike 3.0 United States License.
 * 
 *    http://creativecommons.org/licenses/by-nc-sa/3.0/us/
 * 
 * You are free:
 * 
 *    to Share - to copy, distribute, display, and perform the work
 *    to Remix - to make derivative works
 * 
 * Under the following conditions:
 * 
 *    Attribution. You must attribute the work in the manner specified by the
 *    author or licensor (but not in any way that suggests that they endorse you
 *    or your use of the work).
 * 
 *    Noncommercial. You may not use this work for commercial purposes.
 * 
 *    Share Alike. If you alter, transform, or build upon this work, you may
 *    distribute the resulting work only under the same or similar license to
 *    this one.
 * 
 * For any reuse or distribution, you must make clear to others the license
 * terms of this work. The best way to do this is by including this header.
 * 
 * Any of the above conditions can be waived if you get permission from the
 * copyright holder.
 * 
 * Apart from the remix rights granted under this license, nothing in this
 * license impairs or restricts the author's moral rights.
 */


/**
 * @file
 *
 * @author Joseph Schlecht
 *
 * @cclicense
 *
 * @brief Abstract class to read this object from an input stream.
 */


#ifndef READABLE_H
#define READABLE_H


#include <jwsc++/config.h>

#include <iostream>

#include <jwsc++/base/exception.h>


namespace jwscxx {
namespace base {


/**
 * @class Readable
 *
 * @brief Abstract class to read this object from an input stream.
 */
class Readable
{
    public:

        /** @brief Deletes this Readable. */
        virtual ~Readable() { }


        /** @brief Reads this Readable from an input stream. */
        virtual void read(std::istream& in) throw (jwscxx::base::IO_error,
                jwscxx::base::Arg_error) = 0;


        /** @brief Reads this Readable from a file. */
        virtual void read(const char* fname) throw (jwscxx::base::IO_error,
                jwscxx::base::Arg_error);


        /** 
         * @brief Reads a line off @em in into a buffer and returns a pointer
         * to the field value in the buffer.
         */
        static const char* read_field_value
        (
            std::istream& in,
            const char*   field_name, 
            char*         field_buf,
            size_t        buf_len,
            char          separator=':'
        ) 
        throw (jwscxx::base::Arg_error, jwscxx::base::IO_error);


        /** 
         * @brief Reads a line off @em in into an internal buffer and returns
         * a pointer to the field value in the buffer.
         */
        static const char* read_field_value
        (
            std::istream& in,
            const char*   field_name, 
            char          separator=':'
        ) 
        throw (jwscxx::base::Arg_error, jwscxx::base::IO_error);
};


}}


#endif
