/*
 * This work is licensed under a Creative Commons 
 * Attribution-Noncommercial-Share Alike 3.0 United States License.
 * 
 *    http://creativecommons.org/licenses/by-nc-sa/3.0/us/
 * 
 * You are free:
 * 
 *    to Share - to copy, distribute, display, and perform the work
 *    to Remix - to make derivative works
 * 
 * Under the following conditions:
 * 
 *    Attribution. You must attribute the work in the manner specified by the
 *    author or licensor (but not in any way that suggests that they endorse you
 *    or your use of the work).
 * 
 *    Noncommercial. You may not use this work for commercial purposes.
 * 
 *    Share Alike. If you alter, transform, or build upon this work, you may
 *    distribute the resulting work only under the same or similar license to
 *    this one.
 * 
 * For any reuse or distribution, you must make clear to others the license
 * terms of this work. The best way to do this is by including this header.
 * 
 * Any of the above conditions can be waived if you get permission from the
 * copyright holder.
 * 
 * Apart from the remix rights granted under this license, nothing in this
 * license impairs or restricts the author's moral rights.
 */


/**
 * @file
 *
 * @author Joseph Schlecht
 *
 * @cclicense
 *
 * @brief Abstract class to write this object to an output stream.
 */


#include <jwsc++/config.h>

#include <iostream>
#include <sstream>
#include <fstream>

#include <jwsc++/base/exception.h>
#include <jwsc++/base/writeable.h>

#ifdef JWSCXX_HAVE_DMALLOC
#include <dmalloc.h>
#endif


using namespace jwscxx::base;


/**
 * @param  fname  Output file to write to.
 *
 * @throw  jwscxx::base::IO_error  Could not write to @em fname.
 */
void Writeable::write(const char* fname) const throw (jwscxx::base::IO_error)
{
    std::ofstream out;
    std::ostringstream ost;

    out.open(fname);
    if (out.fail())
    {
        ost << fname << ": Could not open file";
        throw IO_error(ost.str());
    }

    try
    {
        write(out);
    }
    catch (IO_error e)
    {
        ost << fname << ": " << e.get_msg();
        throw IO_error(ost.str());
    }

    out.close();
    if (out.fail())
    {
        ost << fname << ": Could not close file";
        throw IO_error(ost.str());
    }
}
