/*
 * This work is licensed under a Creative Commons 
 * Attribution-Noncommercial-Share Alike 3.0 United States License.
 * 
 *    http://creativecommons.org/licenses/by-nc-sa/3.0/us/
 * 
 * You are free:
 * 
 *    to Share - to copy, distribute, display, and perform the work
 *    to Remix - to make derivative works
 * 
 * Under the following conditions:
 * 
 *    Attribution. You must attribute the work in the manner specified by the
 *    author or licensor (but not in any way that suggests that they endorse you
 *    or your use of the work).
 * 
 *    Noncommercial. You may not use this work for commercial purposes.
 * 
 *    Share Alike. If you alter, transform, or build upon this work, you may
 *    distribute the resulting work only under the same or similar license to
 *    this one.
 * 
 * For any reuse or distribution, you must make clear to others the license
 * terms of this work. The best way to do this is by including this header.
 * 
 * Any of the above conditions can be waived if you get permission from the
 * copyright holder.
 * 
 * Apart from the remix rights granted under this license, nothing in this
 * license impairs or restricts the author's moral rights.
 */


/**
 * @file
 *
 * @author Joseph Schlecht
 *
 * @cclicense
 *
 * @brief Abstract class to write this object to an output stream.
 */


#ifndef WRITEABLE_H
#define WRITEABLE_H


#include <jwsc++/config.h>

#include <iostream>

#include <jwsc++/base/exception.h>


namespace jwscxx {
namespace base {


/**
 * @class Writeable
 *
 * @brief Abstract class to write this object to an output stream.
 */
class Writeable
{
    public:

        /** @brief Deletes this Writeable. */
        virtual ~Writeable() { }


        /** @brief Writes this Writeable to an output stream. */
        virtual void write(std::ostream& out) const 
            throw (jwscxx::base::IO_error) = 0;


        /** @brief Writes this Writeable to a file. */
        virtual void write(const char* fname) const 
            throw (jwscxx::base::IO_error);
};


}}


#endif
