/*
 * This work is licensed under a Creative Commons 
 * Attribution-Noncommercial-Share Alike 3.0 United States License.
 * 
 *    http://creativecommons.org/licenses/by-nc-sa/3.0/us/
 * 
 * You are free:
 * 
 *    to Share - to copy, distribute, display, and perform the work
 *    to Remix - to make derivative works
 * 
 * Under the following conditions:
 * 
 *    Attribution. You must attribute the work in the manner specified by the
 *    author or licensor (but not in any way that suggests that they endorse you
 *    or your use of the work).
 * 
 *    Noncommercial. You may not use this work for commercial purposes.
 * 
 *    Share Alike. If you alter, transform, or build upon this work, you may
 *    distribute the resulting work only under the same or similar license to
 *    this one.
 * 
 * For any reuse or distribution, you must make clear to others the license
 * terms of this work. The best way to do this is by including this header.
 * 
 * Any of the above conditions can be waived if you get permission from the
 * copyright holder.
 * 
 * Apart from the remix rights granted under this license, nothing in this
 * license impairs or restricts the author's moral rights.
 */


/**
 * @file
 *
 * @author Joseph Schlecht
 *
 * @cclicense
 *
 * @brief Abstract class to render this object with the GL.
 */


#ifndef RENDERABLE_H
#define RENDERABLE_H


#include <jwsc++/config.h>

#include <jwsc/matrix/matrix.h>

#include <jwsc++/base/exception.h>


namespace jwscxx {
namespace graphics {


/**
 * @class Renderable
 *
 * @brief Abstract class to render this object with the GL.
 */
class Renderable
{
    public:

        /** @brief Deletes this Renderable. */
        virtual ~Renderable() { }


        /** @brief Renders this object with the GL. */
        virtual void render() const = 0;
};


/**
 * @class Wire_renderable
 *
 * @brief Abstract class to render this object with the GL as a wire-frame.
 */
class Wire_renderable
{
    public:

        /** @brief Deletes this Wire_renderable. */
        virtual ~Wire_renderable() { }


        /** @brief Renders this object with the GL as a wire-frame. */
        virtual void wire_render() const = 0;
};


/**
 * @class Wire_occlude_renderable
 *
 * @brief Abstract class to render this object with the GL as an occluded
 * wire-frame into the depth buffer, to hide unseen lines.
 */
class Wire_occlude_renderable
{
    public:

        /** @brief Deletes this Wire_occlude_renderable. */
        virtual ~Wire_occlude_renderable() { }


        /** 
         * @brief Renders this object with the GL as an occluded wire-frame 
         * into the depth buffer, to hide unseen lines.
         */
        virtual void wire_occlude_render() const = 0;
};


/**
 * @class Solid_renderable
 *
 * @brief Abstract class to render this object with the GL as a solid.
 */
class Solid_renderable
{
    public:

        /** @brief Deletes this Solid_renderable. */
        virtual ~Solid_renderable() { }


        /** @brief Renders this object with the GL as a solid. */
        virtual void solid_render() const = 0;
};


}}


#endif
