/*
 * This work is licensed under a Creative Commons 
 * Attribution-Noncommercial-Share Alike 3.0 United States License.
 * 
 *    http://creativecommons.org/licenses/by-nc-sa/3.0/us/
 * 
 * You are free:
 * 
 *    to Share - to copy, distribute, display, and perform the work
 *    to Remix - to make derivative works
 * 
 * Under the following conditions:
 * 
 *    Attribution. You must attribute the work in the manner specified by the
 *    author or licensor (but not in any way that suggests that they endorse you
 *    or your use of the work).
 * 
 *    Noncommercial. You may not use this work for commercial purposes.
 * 
 *    Share Alike. If you alter, transform, or build upon this work, you may
 *    distribute the resulting work only under the same or similar license to
 *    this one.
 * 
 * For any reuse or distribution, you must make clear to others the license
 * terms of this work. The best way to do this is by including this header.
 * 
 * Any of the above conditions can be waived if you get permission from the
 * copyright holder.
 * 
 * Apart from the remix rights granted under this license, nothing in this
 * license impairs or restricts the author's moral rights.
 */


/**
 * @file
 *
 * @author Joseph Schlecht
 *
 * @cclicense
 *
 * @brief Abstract class to apply a linear transformation to this object.
 */


#ifndef TRANSFORMABLE_H
#define TRANSFORMABLE_H


#include <jwsc++/config.h>

#include <jwsc/matrix/matrix.h>

#include <jwsc++/base/exception.h>


namespace jwscxx {
namespace graphics {


/**
 * @class Transformable_f
 *
 * @brief Abstract class to apply a linear transformation to this object.
 */
class Transformable_f
{
    public:

        /** @brief Deletes this Transformable_f. */
        virtual ~Transformable_f() { }


        /** @brief Transforms this object with a linear operator. */
        virtual void transform(const jwsc::Matrix_f* M) 
            throw (jwscxx::base::Arg_error) = 0;
};


/**
 * @class Transformable_d
 *
 * @brief Abstract class to apply a linear transformation to this object.
 */
class Transformable_d
{
    public:

        /** @brief Deletes this Transformable_d. */
        virtual ~Transformable_d() { }


        /** @brief Transforms this object with a linear operator. */
        virtual void transform(const jwsc::Matrix_d* M) 
            throw (jwscxx::base::Arg_error) = 0;
};


}}


#endif
